@@ -1,4 +1,8 @@
 """Generic Trainer and base configuration for model-based agents."""
+from typing import Dict
+from typing import List
+from typing import Tuple
+
 from ray.rllib import SampleBatch
 from ray.rllib.evaluation.metrics import get_learner_stats
 from ray.rllib.utils import override
@@ -6,27 +10,28 @@ from ray.rllib.utils import override
 from raylab.agents.off_policy import OffPolicyTrainer
 from raylab.agents.off_policy import with_base_config as with_off_policy_config
 from raylab.utils.dictionaries import deep_merge
-from raylab.utils.replay_buffer import ReplayBuffer
+from raylab.utils.replay_buffer import NumpyReplayBuffer
 
 
 BASE_CONFIG = with_off_policy_config(
     {
-        # === Model Training ===
+        # === ModelBasedTrainer ===
+        # === Model Data ===
         # Fraction of replay buffer to use as validation dataset
         # (hence not for training)
         "holdout_ratio": 0.2,
         # Maximum number of samples to use as validation dataset
         "max_holdout": 5000,
+        # === Virtual Replay ===
+        # Size of the buffer for virtual samples
+        "virtual_buffer_size": int(1e6),
+        # number of model rollouts to add to augmented replay per real environment step
+        "model_rollouts": 40,
         # === Policy Training ===
         # Number of policy improvement steps per real environment step
         "policy_improvements": 10,
         # Fraction of each policy minibatch to sample from environment replay pool
         "real_data_ratio": 0.1,
-        # === Replay buffer ===
-        # Size of the buffer for virtual samples
-        "virtual_buffer_size": int(1e6),
-        # number of model rollouts to add to augmented replay per real environment step
-        "model_rollouts": 40,
     }
 )
 
@@ -44,10 +49,9 @@ class ModelBasedTrainer(OffPolicyTrainer):
     @override(OffPolicyTrainer)
     def _init(self, config, env_creator):
         super()._init(config, env_creator)
-
-        self.virtual_replay = ReplayBuffer(
-            config["virtual_buffer_size"], extra_keys=self._extra_replay_keys
-        )
+        policy = self.get_policy()
+        policy.set_reward_from_config(config["env"], config["env_config"])
+        policy.set_termination_from_config(config["env"], config["env_config"])
 
     @staticmethod
     @override(OffPolicyTrainer)
@@ -72,29 +76,50 @@ class ModelBasedTrainer(OffPolicyTrainer):
             config["model_rollouts"] >= 0
         ), "Cannot sample a negative number of model rollouts"
 
+    @override(OffPolicyTrainer)
+    def build_replay_buffer(self, config):
+        super().build_replay_buffer(config)
+        policy = self.get_policy()
+        self.virtual_replay = NumpyReplayBuffer(
+            policy.observation_space, policy.action_space, config["virtual_buffer_size"]
+        )
+        self.virtual_replay.seed(config["seed"])
+
     @override(OffPolicyTrainer)
     def _train(self):
         start_samples = self.sample_until_learning_starts()
 
+        config = self.config
         worker = self.workers.local_worker()
+        policy = worker.get_policy()
+        stats = {}
         while not self._iteration_done():
             samples = worker.sample()
             self.optimizer.num_steps_sampled += samples.count
             for row in samples.rows():
                 self.replay.add(row)
 
-            stats = {}
-            stats.update(self.train_dynamics_model())
-            self.populate_virtual_buffer(samples.count)
-            stats.update(self.improve_policy(samples.count))
+            eval_losses, model_train_info = self.train_dynamics_model()
+            policy.setup_sampling_models(eval_losses)
+            self.populate_virtual_buffer(config["model_rollouts"] * samples.count)
+            policy_train_info = self.improve_policy(
+                config["policy_improvements"] * samples.count
+            )
+
+            stats.update(model_train_info)
+            stats.update(policy_train_info)
 
         self.optimizer.num_steps_sampled += start_samples
         return self._log_metrics(stats)
 
-    def train_dynamics_model(self):
+    def train_dynamics_model(self) -> Tuple[List[float], Dict[str, float]]:
         """Implements the model training step.
 
         Calls the policy to optimize the model on the environment replay buffer.
+
+        Returns:
+            A tuple containing the list of evaluation losses for each model and
+            a dictionary of training statistics
         """
         samples = self.replay.all_samples()
         samples.shuffle()
@@ -104,31 +129,43 @@ class ModelBasedTrainer(OffPolicyTrainer):
         )
         train_data, eval_data = samples.slice(holdout, None), samples.slice(0, holdout)
 
-        policy = self.workers.local_worker().get_policy()
-        stats = get_learner_stats(policy.optimize_model(train_data, eval_data))
+        policy = self.get_policy()
+        eval_losses, stats = policy.optimize_model(train_data, eval_data)
 
-        return stats
+        return eval_losses, stats
 
-    def populate_virtual_buffer(self, num_env_steps):
-        """Add model-generated rollouts branched from real data to the virtual pool."""
-        if not (self.config["model_rollouts"] and self.config["real_data_ratio"] < 1.0):
+    def populate_virtual_buffer(self, num_rollouts: int):
+        """Add model rollouts branched from real data to the virtual pool.
+
+        Args:
+            num_rollouts: Number of initial states to samples from the
+                environment replay buffer
+        """
+        if not (num_rollouts and self.config["real_data_ratio"] < 1.0):
             return
 
-        real_samples = self.replay.sample(self.config["model_rollouts"] * num_env_steps)
-        policy = self.workers.local_worker().get_policy()
+        real_samples = self.replay.sample(num_rollouts)
+        policy = self.get_policy()
         virtual_samples = policy.generate_virtual_sample_batch(real_samples)
         for row in virtual_samples.rows():
             self.virtual_replay.add(row)
 
-    def improve_policy(self, num_env_steps):
-        """Call the policy to perform policy improvement using the augmented replay."""
-        policy = self.workers.local_worker().get_policy()
+    def improve_policy(self, num_improvements: int) -> Dict[str, float]:
+        """Call the policy to perform policy improvement using the augmented replay.
+
+        Args:
+            num_improvements: Number of times to call `policy.learn_on_batch`
+
+        Returns:
+            A dictionary of training and exploration statistics
+        """
+        policy = self.get_policy()
         batch_size = self.config["train_batch_size"]
         env_batch_size = int(batch_size * self.config["real_data_ratio"])
         model_batch_size = batch_size - env_batch_size
 
         stats = {}
-        for _ in range(num_env_steps * self.config["policy_improvements"]):
+        for _ in range(num_improvements):
             samples = []
             if env_batch_size:
                 samples += [self.replay.sample(env_batch_size)]
@@ -137,4 +174,6 @@ class ModelBasedTrainer(OffPolicyTrainer):
             batch = SampleBatch.concat_samples(samples)
             stats = get_learner_stats(policy.learn_on_batch(batch))
             self.optimizer.num_steps_trained += batch.count
+
+        stats.update(policy.get_exploration_info())
         return stats
