@@ -6,7 +6,7 @@ from ray.rllib.utils import override
 from raylab.agents import Trainer
 from raylab.agents import with_common_config
 from raylab.utils.dictionaries import deep_merge
-from raylab.utils.replay_buffer import ReplayBuffer
+from raylab.utils.replay_buffer import NumpyReplayBuffer
 
 BASE_CONFIG = with_common_config(
     {
@@ -34,11 +34,9 @@ class OffPolicyTrainer(Trainer):
     """Generic trainer for off-policy agents."""
 
     # pylint: disable=attribute-defined-outside-init
-
     _name = ""
     _default_config = None
     _policy = None
-    _extra_replay_keys = ()
 
     @override(Trainer)
     def _init(self, config, env_creator):
@@ -46,12 +44,9 @@ class OffPolicyTrainer(Trainer):
         self.workers = self._make_workers(
             env_creator, self._policy, config, num_workers=0
         )
-
         # Dummy optimizer to log stats since Trainer.collect_metrics is coupled with it
         self.optimizer = PolicyOptimizer(self.workers)
-        self.replay = ReplayBuffer(
-            config["buffer_size"], extra_keys=self._extra_replay_keys
-        )
+        self.build_replay_buffer(config)
 
     @override(Trainer)
     def _train(self):
@@ -59,11 +54,13 @@ class OffPolicyTrainer(Trainer):
 
         worker = self.workers.local_worker()
         policy = worker.get_policy()
+        stats = {}
         while not self._iteration_done():
             samples = worker.sample()
             self.optimizer.num_steps_sampled += samples.count
             for row in samples.rows():
                 self.replay.add(row)
+            stats.update(policy.get_exploration_info())
 
             self._before_replay_steps(policy)
             for _ in range(samples.count):
@@ -74,6 +71,14 @@ class OffPolicyTrainer(Trainer):
         self.optimizer.num_steps_sampled += start_samples
         return self._log_metrics(stats)
 
+    def build_replay_buffer(self, config):
+        """Construct replay buffer to hold samples."""
+        policy = self.get_policy()
+        self.replay = NumpyReplayBuffer(
+            policy.observation_space, policy.action_space, config["buffer_size"]
+        )
+        self.replay.seed(config["seed"])
+
     def sample_until_learning_starts(self):
         """
         Sample enough transtions so that 'learning_starts' steps are collected before
