@@ -4,10 +4,11 @@ import torch.nn as nn
 from ray.rllib import SampleBatch
 from ray.rllib.utils import override
 
-import raylab.utils.pytorch as ptu
 from raylab.agents.svg import SVGTorchPolicy
 from raylab.losses import OneStepSVG
 from raylab.policy import AdaptiveKLCoeffMixin
+from raylab.policy import EnvFnMixin
+from raylab.pytorch.optim import get_optimizer_class
 
 
 class SVGOneTorchPolicy(AdaptiveKLCoeffMixin, SVGTorchPolicy):
@@ -21,10 +22,14 @@ class SVGOneTorchPolicy(AdaptiveKLCoeffMixin, SVGTorchPolicy):
             self.module.model.reproduce,
             self.module.actor.reproduce,
             self.module.critic,
-            self.reward,
             gamma=self.config["gamma"],
         )
 
+    @override(EnvFnMixin)
+    def set_reward_from_config(self, env_name: str, env_config: dict):
+        super().set_reward_from_config(env_name, env_config)
+        self.loss_actor.set_reward_fn(self.reward_fn)
+
     @staticmethod
     @override(SVGTorchPolicy)
     def get_default_config():
@@ -42,7 +47,7 @@ class SVGOneTorchPolicy(AdaptiveKLCoeffMixin, SVGTorchPolicy):
     @override(SVGTorchPolicy)
     def make_optimizer(self):
         """PyTorch optimizer to use."""
-        optim_cls = ptu.get_optimizer_class(self.config["torch_optimizer"])
+        optim_cls = get_optimizer_class(self.config["torch_optimizer"])
         options = self.config["torch_optimizer_options"]
         params = [
             dict(params=self.module[k].parameters(), **options[k]) for k in options
