@@ -1,8 +1,10 @@
 """Trainer and configuration for SVG(1) with maximum entropy."""
 from ray.rllib import SampleBatch
+from ray.rllib.utils import override
 
 from raylab.agents.off_policy import OffPolicyTrainer
 from raylab.agents.off_policy import with_base_config
+from raylab.utils.replay_buffer import ReplayField
 
 from .policy import SoftSVGTorchPolicy
 
@@ -64,7 +66,17 @@ DEFAULT_CONFIG = with_base_config(
 class SoftSVGTrainer(OffPolicyTrainer):
     """Single agent trainer for SoftSVG."""
 
+    # pylint: disable=attribute-defined-outside-init
     _name = "SoftSVG"
     _default_config = DEFAULT_CONFIG
     _policy = SoftSVGTorchPolicy
-    _extra_replay_keys = (SampleBatch.ACTION_LOGP,)
+
+    @override(OffPolicyTrainer)
+    def _init(self, config, env_creator):
+        super()._init(config, env_creator)
+        self.get_policy().set_reward_from_config(config["env"], config["env_config"])
+
+    @override(OffPolicyTrainer)
+    def build_replay_buffer(self, config):
+        super().build_replay_buffer(config)
+        self.replay.add_fields(ReplayField(SampleBatch.ACTION_LOGP))
