@@ -2,7 +2,9 @@
 from raylab.utils.dictionaries import deep_merge
 
 from .abstract import AbstractActorCritic
-from .mixins import ActionValueMixin, NormalizingFlowActorMixin, MaximumEntropyMixin
+from .mixins import ActionValueMixin
+from .mixins import MaximumEntropyMixin
+from .mixins import NormalizingFlowActorMixin
 
 
 BASE_CONFIG = {"torch_script": True, "actor": {}, "critic": {}, "entropy": {}}
